@@ -55,6 +55,7 @@ mkDatabase gen connection
 		 dbTables       = hdbcTables          connection,
 		 dbDescribe     = hdbcDescribe        connection,
 		 dbTransaction  = hdbcTransaction     connection,
+                 dbCommit       = HDBC.commit         connection,
 		 dbCreateDB     = hdbcCreateDB    gen connection,
 		 dbCreateTable  = hdbcCreateTable gen connection,
 		 dbDropDB       = hdbcDropDB      gen connection,
@@ -123,7 +124,7 @@ colDescToFieldDesc c = (t, nullable)
             SqlLongVarBinaryT -> string
             SqlDateT          -> CalendarTimeT
             SqlTimeT          -> CalendarTimeT
-            SqlTimestampT     -> CalendarTimeT
+            SqlTimestampT     -> LocalTimeT
             SqlUTCDateTimeT   -> CalendarTimeT
             SqlUTCTimeT       -> CalendarTimeT
             SqlTimeWithZoneT  -> CalendarTimeT
@@ -160,6 +161,9 @@ hdbcTransaction conn action =
 
 type HDBCRow = Map String HDBC.SqlValue
 
+normalizeField :: String -> String
+normalizeField =  map toLower
+
 -- | Primitive query
 hdbcPrimQuery :: (GetRec er vr, IConnection conn) => 
 		 conn -- ^ Database connection.
@@ -171,8 +175,13 @@ hdbcPrimQuery conn sql scheme rel =
     do
     stmt <- handleSqlError $ HDBC.prepare conn sql
     handleSqlError $ HDBC.execute stmt []
-    rows <- HDBC.fetchAllRowsMap stmt
-    mapM (getRec hdbcGetInstances rel scheme) rows
+    rows <- fetchNormalizedAllRowsAL stmt
+    mapM (getRec hdbcGetInstances rel scheme) $ map Map.fromList rows
+  where fetchNormalizedAllRowsAL sth =
+          do
+          names <- map normalizeField `fmap` getColumnNames sth
+          rows <- fetchAllRows sth
+          return $ map (zip names) rows
 
 -- | Primitive execute
 hdbcPrimExecute :: (IConnection conn) => conn -- ^ Database connection.
@@ -197,10 +206,11 @@ hdbcGetInstances =
 		 , getDouble       = hdbcGetValue
 		 , getBool         = hdbcGetValue
 		 , getCalendarTime = hdbcGetValue
+		 , getLocalTime    = hdbcGetValue
 		 }
 
 -- hdbcGetValue :: Data.Convertible.Base.Convertible SqlValue a
 --             => HDBCRow -> String -> IO (Maybe a)
-hdbcGetValue m f = case Map.lookup (map toLower f) m of
+hdbcGetValue m f = case Map.lookup (normalizeField f) m of
                      Nothing -> fail $ "No such field " ++ f
                      Just x  -> return $ HDBC.fromSql x
